'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireDefault(_cheerio);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Nasdaq = function () {
  function Nasdaq(config) {
    (0, _classCallCheck3.default)(this, Nasdaq);

    this.config = _lodash2.default.defaults(config, {
      requestConcurrency: 4,
      requestDelay: 0
    });

    this.request = _requestPromise2.default;
  }

  (0, _createClass3.default)(Nasdaq, [{
    key: '_getDataUrl',
    value: function _getDataUrl(ticker) {
      return 'http://www.nasdaq.com/g00/symbol/' + ticker + '/time-sales';
    }
  }, {
    key: '_getTickerPage',
    value: function _getTickerPage(ticker) {
      var postfix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

      var uri = this._getDataUrl(ticker) + postfix;
      return this.request({
        uri: uri,
        transform: function transform(body) {
          return _cheerio2.default.load(body);
        }
      });
    }

    /**
     * Load price from a cheerio object
     * @param $ Cheerio object
     * @returns {null|number}
     * @private
     */

  }, {
    key: '_parsePrice',
    value: function _parsePrice($) {
      return Number($('#qwidget_lastsale').text().replace('$', ''));
    }

    /**
     * Load info about given ticker
     * @param ticker Ticker eg tsla
     * @returns {Promise} Resolves into an object with ticker info
     */

  }, {
    key: 'getInfo',
    value: function getInfo(ticker) {
      var _this = this;

      return this._getTickerPage(ticker).then(function ($) {
        return {
          name: $('.greenCompanyName').text(),
          exchange: $('#qbar_exchangeLabel').text().replace('Exchange:', ''),
          industry: $('#qbar_sectorLabel a').text(),
          image: $('#logo-wrap img').attr('x-defer-src'),
          price: _this._parsePrice($),
          priceChange: $('#qwidget_netchange').text(),
          priceChangePercent: $('#qwidget_percent').text().replace('%', '')
        };
      });
    }

    /**
     * Load price for a given ticker
     * @param ticker
     * @returns {Promise.<TResult>}
     */

  }, {
    key: 'getPrice',
    value: function getPrice(ticker) {
      var _this2 = this;

      return this._getTickerPage(ticker).then(function ($) {
        return _this2._parsePrice($);
      });
    }
  }, {
    key: '_parseTicks',
    value: function _parseTicks($) {
      var rows = $('#AfterHoursPagingContents_Table tr');
      var ticks = [];

      rows.each(function (index, row) {
        if (!index) return; // skip header row

        var cells = (0, _cheerio2.default)(row);
        var tick = {
          time: _lodash2.default.trim(cells.find('td:nth-child(1)').text()),
          price: _lodash2.default.trim(cells.find('td:nth-child(2)').text().replace('$', '')),
          volume: cells.find('td:nth-child(3)').text()
        };
        ticks.push(tick);
      });

      return ticks;
    }
  }, {
    key: '_getLastPageNumber',
    value: function _getLastPageNumber($) {
      var href = $('#quotes_content_left_lb_LastPage').attr('href');
      var res = (href || '').match(/pageno=([1-9]+)/);
      return res && res.length >= 2 ? Number(res[1]) : null;
    }
  }, {
    key: '_loadPageTicks',
    value: function _loadPageTicks(scheduler, ticker, sectionNumber, pageNumber) {
      var _this3 = this;

      var postfix = '?time=' + sectionNumber + '&pageno=' + pageNumber;
      var pageId = this._getPageId(sectionNumber, pageNumber);

      if (scheduler.lastValidPageId < pageId) return _bluebird2.default.resolve([]);

      console.log('Downloading %s - section %d - page %d', ticker, sectionNumber, pageNumber);
      return _bluebird2.default.resolve(this._getTickerPage(ticker, postfix)).delay(this.config.requestDelay).then(function ($) {
        return _this3._parseTicks($);
      }).then(function (ticks) {
        if (ticks.length === 0) scheduler.lastValidPageId = pageId;
        return ticks.reverse();
      });
    }
  }, {
    key: '_loadSection',
    value: function _loadSection(scheduler, ticker, sectionNumber) {
      var _this4 = this;

      var ticks = [];
      var pageId = this._getPageId(sectionNumber);

      if (scheduler.lastValidPageId < pageId) return _bluebird2.default.resolve([]);

      // fetch first page
      return this._getTickerPage(ticker, '?time=' + sectionNumber).then(function ($) {
        var lastPage = _this4._getLastPageNumber($);
        var range = _lodash2.default.range(2, lastPage + 1);
        ticks.push(_this4._parseTicks($));

        if (ticks.length === 0) {
          scheduler.lastValidPageId = pageId;
          return _bluebird2.default.resolve([]);
        }

        return _bluebird2.default.map(range, function (pageNumber) {
          return _this4._loadPageTicks(scheduler, ticker, sectionNumber, pageNumber);
        }, { concurrency: _this4.config.requestConcurrency }).then(function (data) {
          return ticks.push.apply(ticks, (0, _toConsumableArray3.default)(data)) && _lodash2.default.flatten(ticks.reverse());
        });
      });
    }
  }, {
    key: '_getPageId',
    value: function _getPageId(section) {
      var page = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      return section * 100000 + page;
    }
  }, {
    key: 'getTicks',
    value: function getTicks(ticker) {
      var _this5 = this;

      var range = _lodash2.default.range(1, 14);
      var scheduler = {
        lastValidPageId: Number.MAX_VALUE
      };

      return _bluebird2.default.map(range, function (section) {
        return _this5._loadSection(scheduler, ticker, section);
      }, { concurrency: 1 }).then(function (ticks) {
        return _lodash2.default.flatten(ticks);
      });
    }
  }]);
  return Nasdaq;
}();

exports.default = Nasdaq;